'use client'
import { createAssignPriorityRequest, deleteAssignPriority, editAssignPriorityRequest } from "@/app/lib/assignment-priority/action";
import { createIbcRequest } from "@/app/lib/ibc/action";
import DataTable from "@/components/tables/DataTable";
import { DeleteIcon, EditIcon } from "@chakra-ui/icons";
import { Badge, Button, Flex, Icon, useColorModeValue, useDisclosure, useToast } from "@chakra-ui/react";
import { useRouter } from "next/navigation";
import { useState } from "react";
import { AssignPriorityInterface } from "../type/ibc";
import { columnsDataBlocks } from "../variables/columnsBlock";
import IbcModalForm from "./form";
import useShowToast from "@/app/hooks/useShowToast";
import { BadgeColorKey, badgeColors } from "@/utils/constant";
import { useGeneral } from "@/contexts/GeneralContext";
import { hasPermission } from "@/utils/navigation";



export function CustomBadge({ color, title }: { color: BadgeColorKey; title?: string; }) {
  const bg = useColorModeValue(badgeColors[color]?.bg, badgeColors[color]?.bgDark);
  const text = useColorModeValue(badgeColors[color]?.text, badgeColors[color]?.textDark);

  return (
    <>
      <Badge bg={bg} color={text}>
        {title ?? color}
      </Badge>
    </>
  );
}


interface AssignPriorityTableProps {
  data: { data: AssignPriorityInterface[]; }
}

const AssignmentPriorityTable: React.FC<AssignPriorityTableProps> = ({ data }) => {
  const { currentUser } = useGeneral();
  const { isOpen, onOpen, onClose } = useDisclosure();
  const showToast = useShowToast();
  const [selectedIbc, setSelectedIbc] = useState<AssignPriorityInterface | null>(null);
  const toast = useToast();
  const router = useRouter();

  const deleteUser = async (id: number) => {
    const res = await deleteAssignPriority(id);
    if (res?.status == 200) {
      showToast("Deleted!", res?.message, "success");
      router.refresh();
    } else {
      showToast("Failed to Delete", res?.message, "error");
    }
  };

  const handleAddClick = () => {
    setSelectedIbc(null); // Reset for add
    onOpen();
  };

  const handleEditClick = (user: number) => {
    const singleUser = data?.data?.find((item: any) => (item.id == user));
    setSelectedIbc((prev) => {
      return { ...singleUser };
    });
    onOpen();
  };

  const handleSubmit = async (values: FormData) => {
    if (selectedIbc) {
      const payload = { userId: selectedIbc.id, data: values };
      const res = await editAssignPriorityRequest(payload);
      if (res?.status == 200) {
        toast({
          title: "Success",
          description: res.message,
          status: "success",
          position: "top-right",
          duration: 2000,
          isClosable: true,
        });
        onClose();
        router.refresh();
      } else {
        toast({
          title: "Error",
          description: res.message,
          status: "error",
          position: "top-right",
          duration: 9000,
          isClosable: true,
        });
      }
    } else {
      const res: any = await createAssignPriorityRequest(values);
      if (res?.status == 200) {
        toast({
          title: "Success",
          description: res.message,
          status: "success",
          position: "top-right",
          duration: 2000,
          isClosable: true,
        });
        onClose();
        router.refresh();
      } else {
        toast({
          title: "Error",
          description: res.message,
          status: "error",
          position: "top-right",
          duration: 9000,
          isClosable: true,
        });
      }
    }
  };

  const action = (id: number) => (
    <Flex gap={2}>
      {hasPermission(currentUser.permissions, "setting.assignment-priority", "edit") && (
        <Button
          variant="blank"
          onClick={() => handleEditClick(id)}
          w="37px"
          h="37px"
          borderRadius="10px">
          <Icon as={EditIcon} color="brand.500" w="20px" h="20px" />
        </Button>
      )}
      {hasPermission(currentUser.permissions, "setting.assignment-priority", "delete") && (
        <Button
          variant="blank"
          onClick={() => deleteUser(id)}
          w="37px"
          h="37px"
          borderRadius="10px">
          <Icon as={DeleteIcon} color="brand.500" w="20px" h="20px" />
        </Button>
      )}
    </Flex>
  );

  return (
    <>
      <DataTable
        addButton={hasPermission(currentUser.permissions, "setting.assignment-priority", "create") && <Button variant="solid" colorScheme="blue" onClick={handleAddClick}>Create</Button>}
        columnsData={columnsDataBlocks}
        tableData={data?.data as unknown as AssignPriorityInterface[]}
        paginate={data}
        isSelectable={false}
        showPageSize={false}
        actionSlot={(user: number) => action(user)}
      />

      <IbcModalForm
        isOpen={isOpen}
        onClose={onClose}
        currentUser={selectedIbc}
        handleSubmit={handleSubmit}
      />
    </>
  );
};

export default AssignmentPriorityTable;
